@@ -25,6 +25,7 @@ void xapi_close (void *d);
 extern double framerate;
 
 #import <Cocoa/Cocoa.h>
+#include <libgen.h>
 #include <pthread.h>
 
 static pthread_mutex_t osx_vbuf_lock = PTHREAD_MUTEX_INITIALIZER;
@@ -39,6 +40,9 @@ static volatile uint8_t sync_sem;
 static void gl_sync_lock ()   { if (osxgui_status > 0) { sync_sem = 1; pthread_mutex_lock (&osxgui_sync); }}
 static void gl_sync_unlock () { if (osxgui_status > 0) { sync_sem = 0; pthread_mutex_unlock (&osxgui_sync); }}
 
+static void gl_newsrc_sync () ;
+static bool _newsrc = false;
+
 #define PTLL gl_sync_lock()
 #define PTUL gl_sync_unlock()
 
@@ -190,6 +194,7 @@ __attribute__ ((visibility ("hidden")))
 - (void) drawRect:(NSRect)rect
 {
 	pthread_mutex_lock(&osx_vbuf_lock);
+	if (_newsrc) { gl_newsrc_sync(); }
 	xjglExpose(osx_vbuf);
 	pthread_mutex_unlock(&osx_vbuf_lock);
 }
@@ -302,11 +307,9 @@ __attribute__ ((visibility ("hidden")))
 
 - (void) keyDown:(NSEvent*)event
 {
-	if (!([event isARepeat])) {
-		NSString* chars = [event characters];
-		char buf[2] = {[chars characterAtIndex:0], 0};
-		xjglKeyPress([chars characterAtIndex:0], buf);
-	}
+	NSString* chars = [event characters];
+	char buf[2] = {[chars characterAtIndex:0], 0};
+	xjglKeyPress([chars characterAtIndex:0], buf);
 }
 
 - (void) setFullScreen:(int)onoff
@@ -330,6 +333,7 @@ static id osx_window;
 // Menus
 static NSMenuItem *mFileOpen;
 static NSMenuItem *mFileClose;
+static NSMenuItem *mFileRecent;
 static NSMenuItem *mJackTransport;
 static NSMenuItem *mSyncJACK;
 #ifdef HAVE_LTC
@@ -351,7 +355,8 @@ static NSMenuItem *mOsdOffOff;
 static NSMenuItem *mOsdOffTc;
 static NSMenuItem *mOsdOffFn;
 static NSMenuItem *mOsdBox;
-static NSMenuItem *mOsdNfo;
+static NSMenuItem *mOsdTNfo;
+static NSMenuItem *mOsdGNfo;
 static NSMenuItem *mOsdGeo;
 
 static NSMenuItem *mDpyLetterbox;
@@ -469,8 +474,10 @@ static void update_osd_menu () {
 	[mOsdOffTc  setState:(OSD_mode & OSD_OFFS)  ? NSOnState : NSOffState];
 	[mOsdOffFn  setState:(OSD_mode & OSD_OFFF)  ? NSOnState : NSOffState];
 	[mOsdBox    setState:(OSD_mode & OSD_BOX)   ? NSOnState : NSOffState];
-	[mOsdNfo    setState:(OSD_mode & OSD_NFO)   ? NSOnState : NSOffState];
-	[mOsdNfo    setEnabled:(movie_height < OSD_MIN_NFO_HEIGHT) ? NO : YES];
+	[mOsdTNfo   setState:(OSD_mode & OSD_NFO)   ? NSOnState : NSOffState];
+	[mOsdGNfo   setState:(OSD_mode & OSD_GEO)   ? NSOnState : NSOffState];
+	[mOsdTNfo   setEnabled:(movie_height < OSD_MIN_NFO_HEIGHT) ? NO : YES];
+	[mOsdGNfo   setEnabled:(movie_height < OSD_MIN_NFO_HEIGHT) ? NO : YES];
 	[mOsdGeo    setState:(OSD_mode & OSD_GEO)   ? NSOnState : NSOffState];
 	[mOsdGeo    setEnabled:(movie_height < OSD_MIN_NFO_HEIGHT) ? NO : YES];
 }
@@ -494,9 +501,27 @@ static void update_dpy_menu () {
 			if (interaction_override&OVR_LOADFILE) {
 				[mFileOpen setEnabled:NO];
 				[mFileClose setEnabled:NO];
+				[mFileRecent setEnabled:NO];
 			} else {
+				unsigned int i, recent = x_fib_recent_count();
 				[mFileOpen setEnabled:YES];
 				[mFileClose setEnabled:  have_open_file() ? YES : NO];
+				[mFileRecent setEnabled: recent > 0 ? YES : NO];
+
+				NSMenu *recentMenu = [[NSMenu alloc] initWithTitle:@"Recently Used"];
+				for (i = 0; i < recent && i < 8; ++i) {
+					if (!x_fib_recent_at (i)) break;
+					char *tmp = strdup (x_fib_recent_at (i));
+					char *base = basename(tmp);
+					if (base) {
+						NSString *name = [NSString stringWithUTF8String:base];
+						NSMenuItem *mi = [recentMenu addItemWithTitle:name action:@selector(openRecent:) keyEquivalent:@""];
+						[mi setRepresentedObject: [NSNumber numberWithInt:i]];
+					}
+					free(tmp);
+				}
+				[mFileRecent setSubmenu:recentMenu];
+				[recentMenu release];
 			}
 			break;
 		case 2:
@@ -575,6 +600,16 @@ static void update_dpy_menu () {
 	PTLL; xapi_close (NULL); PTUL;
 }
 
+- (void) openRecent: (id)sender
+{
+	if (interaction_override&OVR_LOADFILE) return;
+	int selectedItem = [[sender representedObject] intValue];
+	const char *fn = x_fib_recent_at (selectedItem);
+	if (fn) {
+		PTLL; xapi_open ((void*)fn); PTUL;
+	}
+}
+
 - (void) syncJack: (id)sender { PTLL; ui_sync_to_jack(); PTUL; }
 - (void) syncLTC: (id)sender  { PTLL; ui_sync_to_ltc(); PTUL; }
 - (void) syncMTCJ: (id)sender { PTLL; ui_sync_to_mtc_jack(); PTUL; }
@@ -630,13 +665,18 @@ static void gl_swap_buffers () {
 
 void gl_newsrc () {
 	pthread_mutex_lock (&osx_vbuf_lock);
+	_newsrc = true;
+	pthread_mutex_unlock (&osx_vbuf_lock);
+}
+
+static void gl_newsrc_sync () {
 	free (osx_vbuf);
 	osx_vbuf_size = video_buffer_size ();
 	if (osx_vbuf_size > 0) {
 		osx_vbuf = (uint8_t*)malloc (osx_vbuf_size * sizeof(uint8_t));
 		gl_reallocate_texture (movie_width, movie_height);
 	}
-	pthread_mutex_unlock (&osx_vbuf_lock);
+	_newsrc = false;
 }
 
 static void makeAppMenu (void) {
@@ -686,6 +726,10 @@ static void makeAppMenu (void) {
 	mFileOpen  = [fileMenu addItemWithTitle:@"Open" action:@selector(openVideo:) keyEquivalent:@"o"];
 	mFileClose = [fileMenu addItemWithTitle:@"Close" action:@selector(closeVideo:) keyEquivalent:@"w"];
 
+	mFileRecent = [[NSMenuItem alloc] initWithTitle:@"Recent" action:nil keyEquivalent:@""];
+	[fileMenu addItem:[NSMenuItem separatorItem]];
+	[fileMenu addItem:mFileRecent];
+
 	fileMenuItem = [[NSMenuItem alloc] initWithTitle:@"File" action:nil keyEquivalent:@""];
 	[fileMenuItem setSubmenu:fileMenu];
 	[[NSApp mainMenu] addItem:fileMenuItem];
@@ -718,7 +762,7 @@ static void makeAppMenu (void) {
 	[syncMenu addItem:[NSMenuItem separatorItem]];
 
 	NSMenu     *jackMenu;
-	jackMenu = [[NSMenu alloc] initWithTitle:@"Jack Transport"];
+	jackMenu = [[NSMenu alloc] initWithTitle:@"JACK Transport"];
 
 	menuItem = [jackMenu addItemWithTitle:@"Play/Pause" action:@selector(jackPlayPause:) keyEquivalent:@" "];
 	[menuItem setKeyEquivalentModifierMask:0];
@@ -727,7 +771,7 @@ static void makeAppMenu (void) {
 	menuItem = [jackMenu addItemWithTitle:@"Rewind" action:@selector(jackRewind:) keyEquivalent:@"\010"];
 	[menuItem setKeyEquivalentModifierMask:0];
 
-	mJackTransport = [[NSMenuItem alloc] initWithTitle:@"Jack Transport" action:nil keyEquivalent:@""];
+	mJackTransport = [[NSMenuItem alloc] initWithTitle:@"JACK Transport" action:nil keyEquivalent:@""];
 	[mJackTransport setSubmenu:jackMenu];
 	[syncMenu addItem:mJackTransport];
 	[jackMenu release];
@@ -758,20 +802,21 @@ static void makeAppMenu (void) {
 	mOsdOffTc  = [osdMenu addItemWithTitle:@"Offset Timecode" action:@selector(osdOffTc:) keyEquivalent:@""];
 	mOsdOffFn  = [osdMenu addItemWithTitle:@"Offset Frame Number" action:@selector(osdOffFn:) keyEquivalent:@""];
 	[osdMenu addItem:[NSMenuItem separatorItem]];
-	mOsdNfo   = [osdMenu addItemWithTitle:@"Time Info" action:@selector(osdNfo:) keyEquivalent:@"i"];
-	mOsdNfo   = [osdMenu addItemWithTitle:@"Geometry" action:@selector(osdGeo:) keyEquivalent:@"g"];
+	mOsdTNfo   = [osdMenu addItemWithTitle:@"Time Info" action:@selector(osdNfo:) keyEquivalent:@"i"];
+	mOsdGNfo   = [osdMenu addItemWithTitle:@"Geometry" action:@selector(osdGeo:) keyEquivalent:@"g"];
 	[osdMenu addItem:[NSMenuItem separatorItem]];
-	mOsdBox   = [osdMenu addItemWithTitle:@"Background" action:@selector(osdBox:) keyEquivalent:@"b"];
-	menuItem  = [osdMenu addItemWithTitle:@"Swap Position" action:@selector(osdPos:) keyEquivalent:@"p"];
+	mOsdBox    = [osdMenu addItemWithTitle:@"Background" action:@selector(osdBox:) keyEquivalent:@"b"];
+	menuItem   = [osdMenu addItemWithTitle:@"Swap Position" action:@selector(osdPos:) keyEquivalent:@"p"];
 	[menuItem   setKeyEquivalentModifierMask:0];
 	[osdMenu addItem:[NSMenuItem separatorItem]];
-	menuItem  = [osdMenu addItemWithTitle:@"Clear All" action:@selector(osdClear:) keyEquivalent:@"C"];
+	menuItem   = [osdMenu addItemWithTitle:@"Clear All" action:@selector(osdClear:) keyEquivalent:@"C"];
 	[menuItem   setKeyEquivalentModifierMask:NSShiftKeyMask];
 
 	[menuItem   setKeyEquivalentModifierMask:0];
 	[mOsdVtcOff setKeyEquivalentModifierMask:0];
 	[mOsdOffOff setKeyEquivalentModifierMask:0];
-	[mOsdNfo    setKeyEquivalentModifierMask:0];
+	[mOsdGNfo   setKeyEquivalentModifierMask:0];
+	[mOsdTNfo   setKeyEquivalentModifierMask:0];
 	[mOsdGeo    setKeyEquivalentModifierMask:0];
 	[mOsdBox    setKeyEquivalentModifierMask:0];
 	[mOsdTC     setKeyEquivalentModifierMask:0];
@@ -915,7 +960,10 @@ static int osx_open_window () {
 		osx_close_window ();
 		return -1;
 	}
-	gl_newsrc ();
+
+	pthread_mutex_lock (&osx_vbuf_lock);
+	gl_newsrc_sync ();
+	pthread_mutex_unlock (&osx_vbuf_lock);
 
 	[window setIsVisible:YES];
 
@@ -1015,6 +1063,8 @@ void osx_main () {
 						break;
 					case 3:
 						pthread_mutex_lock (&osx_vbuf_lock);
+						if (_newsrc) { gl_newsrc_sync(); }
+						force_redraw = 1;
 						xjglExpose (osx_vbuf);
 						pthread_mutex_unlock (&osx_vbuf_lock);
 						[osx_glview setNeedsDisplay:YES];
@@ -1051,7 +1101,11 @@ void osx_main () {
 void gl_render (uint8_t *buffer) {
 	if (!osx_vbuf) return;
 	pthread_mutex_lock (&osx_vbuf_lock);
-	memcpy (osx_vbuf, buffer, osx_vbuf_size * sizeof(uint8_t));
+	if (_newsrc) {
+		osx_post_event (1);
+	} else {
+		memcpy (osx_vbuf, buffer, osx_vbuf_size * sizeof(uint8_t));
+	}
 	pthread_mutex_unlock (&osx_vbuf_lock);
 	[osx_glview setNeedsDisplay:YES];
 }
