@@ -39,6 +39,73 @@ void osx_main ();
 void osx_shutdown();
 #endif
 
+// configuration strings
+char const * const cfg_features = ""
+#define CFG_STRING ""
+#ifdef HAVE_LTC
+"LTC "
+#endif
+#ifdef JACK_SESSION
+	"JACK-SESSION "
+#endif
+#ifdef HAVE_MQ
+	"POSIX-MQueue "
+#elif defined HAVE_IPCMSG
+	"IPC-MSG "
+#endif
+#ifdef HAVE_LIBLO
+	"OSC "
+#endif
+	;
+
+char const * const cfg_midi = ""
+#ifndef HAVE_MIDI
+	"*disabled*"
+#else /* have Midi */
+# ifdef HAVE_JACKMIDI
+	"jack-midi "
+# endif
+# ifdef ALSA_SEQ_MIDI
+	"alsa-sequencer "
+# endif
+# ifdef HAVE_PORTMIDI
+	"portmidi "
+# endif
+# ifdef ALSA_RAW_MIDI
+	"alsa-raw "
+# endif
+#endif /* HAVE_MIDI */
+	;
+
+char const * const cfg_displays = ""
+#if HAVE_GL
+	"openGL "
+#endif
+#if HAVE_LIBXV
+	"Xv "
+#endif
+#ifdef HAVE_SDL
+	"SDL "
+#endif
+#if HAVE_IMLIB2
+	"X11/imlib2"
+# ifdef IMLIB2RGBA
+	"(RGBA32) "
+# else
+	"(RGB24) "
+# endif
+#endif
+#ifdef PLATFORM_OSX
+	"OSX/quartz "
+#endif
+	;
+
+#ifndef CFG_COMPAT
+#define CFG_COMPAT "xjadeo1"
+#endif
+
+char cfg_compat[1024] = CFG_COMPAT;
+
 //------------------------------------------------
 // Globals
 //------------------------------------------------
@@ -541,10 +608,10 @@ usage (int status)
 "                           Control. For a list of available commands, please\n"
 "                           see the source code or online documentation.\n"
 " -o <val>, --offset <val>\n"
-"                           Time offset video from timecode source.\n"
-"                           This allows to offset the video playback a certain\n"
-"                           number of (video) frames relative to the time\n"
-"                           source. Positive and negative values are permitted.\n"
+"                           Time offset video from timecode source. This allows\n"
+"                           one to offset the video playback a certain number\n"
+"                           of (video) frames relative to the time source.\n"
+"                           Positive and negative values are permitted.\n"
 "                           The offset van be specified either as integer frame\n"
 "                           number of as colon separated timecode.\n"
 #if 0
@@ -554,6 +621,7 @@ usage (int status)
 "                           Note that this may requires parsing future frames\n"
 "                           and impact performance\n"
 #endif
+" --osc-doc                 Print available OSC commands and exit.\n"
 " -Q, --mq                  Enable POSIX realtime message queues.\n"
 "                           This sets up a communication channel for remote,\n"
 "                           intended to be used with `xjremote`.\n"
@@ -675,7 +743,7 @@ usage (int status)
 " '0-9'        Change color equalization (x11/imlib and XV only)\n"
 " 'Shift+1-4'  Fine tune color equalization (x11/imlib and XV only)\n"
 "              brightness:1+2, contrast:3+4, gamma:5+6, saturation:7+8\n"
-"              hue:9+0. XV color balance is hardware dependant.\n"
+"              hue:9+0. XV color balance is hardware dependent.\n"
 "\n"
 "Note that it is possible to disable certain features using the remote control.\n"
 "A Host can take control of certain aspects e.g. offset or disallow closing the\n"
@@ -694,72 +762,14 @@ static void printversion (void) {
 
 #ifdef SUBVERSION
 	if (strlen(SUBVERSION)>0 && strcmp(SUBVERSION, "exported")) {
-		printf (" built from:     scm-%s\n", SUBVERSION);
+		printf (" built from:    scm-%s\n", SUBVERSION);
 	}
 #endif
-	printf(" compiled with:  AVFORMAT=0x%x AVCODEC=0x%x AVUTIL:0x%x\n",
+	printf (" compiled with: AVFORMAT=0x%x AVCODEC=0x%x AVUTIL:0x%x\n",
 			LIBAVFORMAT_VERSION_INT, LIBAVCODEC_VERSION_INT, LIBAVUTIL_VERSION_INT);
-	printf (" configuration:  ");
-	printf ("[ ");
-#ifdef HAVE_LTC
-	printf("LTC ");
-#endif
-#ifdef JACK_SESSION
-	printf("JACK-SESSION ");
-#endif
-#ifdef HAVE_MQ
-	printf("POSIX-MQueue ");
-#elif defined HAVE_IPCMSG
-	printf("IPC-MSG ");
-#endif
-#ifdef HAVE_LIBLO
-	printf("OSC ");
-#endif
-	printf("]\n");
-
-	printf(" MTC/MIDI:       ");
-#ifndef HAVE_MIDI
-	printf("*disabled*\n");
-#else /* have Midi */
-	printf("[ ");
-# ifdef HAVE_JACKMIDI
-	printf("jack-midi ");
-# endif
-# ifdef ALSA_SEQ_MIDI
-	printf("alsa-sequencer ");
-# endif
-# ifdef HAVE_PORTMIDI
-	printf("portmidi ");
-# endif
-# ifdef ALSA_RAW_MIDI
-	printf("alsa-raw ");
-# endif
-	printf("]\n");
-#endif /* HAVE_MIDI */
-
-	printf(" Display(s):     [ "
-#if HAVE_GL
-			"openGL "
-#endif
-#if HAVE_LIBXV
-			"Xv "
-#endif
-#ifdef HAVE_SDL
-			"SDL "
-#endif
-#if HAVE_IMLIB2
-			"X11/imlib2"
-# ifdef IMLIB2RGBA
-			"(RGBA32) "
-# else
-			"(RGB24) "
-# endif
-#endif
-#ifdef PLATFORM_OSX
-			"OSX/quartz "
-#endif
-			"]\n"
-			);
+	printf (" configuration: [ %s]\n", cfg_features);
+	printf (" MTC/MIDI:      [ %s]\n", cfg_midi);
+	printf (" Display(s):    [ %s]\n", cfg_displays);
 	printf ("\n"
 			"Copyright (C) GPL 2006-2014 Robin Gareus <robin@gareus.org>\n"
 			"Copyright (C) GPL 2006-2011 Luis Garrido <luisgarrido@users.sourceforge.net>\n"
@@ -821,6 +831,9 @@ static void clean_up (int status) {
 	if (load_rc) free(load_rc);
 	if (load_movie) free(load_movie);
 
+	x_fib_save_recent (x_fib_recent_file ("xjadeo"));
+	x_fib_free_recent ();
+
 	if (!want_quiet)
 		fprintf(stdout, "\nBye!\n");
 	exit(status);
@@ -1014,6 +1027,8 @@ int main (int argc, char **argv)
 
 	if (videomode < 0) vidoutmode(videomode); // dump modes and exit.
 
+	x_fib_load_recent (x_fib_recent_file ("xjadeo"));
+
 	if ((i+1)== argc) movie = argv[i];
 	else movie = "";
 	// NB without WINMENU or XDND as well with SDL
