@@ -99,6 +99,7 @@ extern char OSD_msg[128];
 extern char OSD_nfo_tme[5][48];
 extern char OSD_nfo_geo[5][48];
 uint64_t    osd_smpte_ts;
+uint64_t    osd_vtc_oob;
 
 //------------------------------------------------
 // globals
@@ -236,6 +237,7 @@ void event_loop (void) {
 	splash_timeout = clock1 + 2500000; // 2.5 sec;
 
 	while (loop_flag) { /* MAIN LOOP */
+		uint8_t we_know_transport_is_not_rolling = 0;
 
 		if (loop_run == 0) {
 			/* video offline - (eg. window minimized)
@@ -256,8 +258,11 @@ void event_loop (void) {
 		else
 #endif
 		{
-			newFrame = jack_poll_frame();
+			uint8_t jack_rolling = 1;
+			newFrame = jack_poll_frame(&jack_rolling);
 			syncnidx = 1;
+			if (!jack_rolling)
+				we_know_transport_is_not_rolling = 1;
 		}
 
 		if (newFrame < 0) {
@@ -268,9 +273,48 @@ void event_loop (void) {
 		if (prev_syncidx != syncnidx) {
 			force_redraw = 1;
 			osd_smpte_ts = -1;
+			osd_vtc_oob  = -1;
 			prev_syncidx = syncnidx;
 		}
 
+#if 0 // experimental PLL
+		static uint8_t dll_initialized = 0;
+		static double dll_e2, dll_e0;
+		static double dll_t0, dll_t1;
+		static double dll_b, dll_c;
+		static double dll_frameno;
+		static int64_t prevFrame = 0;
+		if (!scan_complete) dll_initialized = 0;
+		if (syncnidx != 3) {
+			// USE DLL to smooth over large jack cycles
+			// with jack -> we know transport rolling state -> use it
+			// this still jitters on start/stop,:( more work is needed.
+			if (newFrame < prevFrame || newFrame > prevFrame + 4 || we_know_transport_is_not_rolling || !dll_initialized) {
+				// reset DLL
+				dll_initialized = 1;
+				dll_e0 = dll_t0 = 0;
+				dll_e2 = (delay > 0) ? (.5 * framerate * (float)delay) : .2;
+				dll_t1 = newFrame + dll_e2;
+
+				const double omega = 2. * M_PI / framerate;
+				dll_b = 1.4142135623730950488 * omega; // sqrt(2)
+				dll_c = omega * omega;
+				if (!want_quiet && !we_know_transport_is_not_rolling)
+					printf("RE-INIT DLL %g %lld <> %lld\n", dll_e2, prevFrame, newFrame);
+				prevFrame = newFrame;
+			} else {
+				double expect = prevFrame + (xj_get_monotonic_time() - clock1) * framerate / 1000000.f;
+				dll_e0 = expect - dll_t1;
+				dll_t0 = dll_t1;
+				dll_t1 += dll_b * dll_e0 + dll_e2;
+				dll_e2 += dll_c * dll_e0;
+				//printf("%.1f %+7.4f %lld %s\n", dll_t1, dll_e0, newFrame,  floor(dll_t1) != newFrame ? "!":"");
+				prevFrame = newFrame;
+				newFrame = floor(dll_t1);
+			}
+		}
+#endif
+
 #if 0 // DEBUG
 		static int64_t oldFrame = 0;
 		if (oldFrame != newFrame) {
@@ -337,8 +381,8 @@ void event_loop (void) {
 #if 0 // debug timing
 			printf("  %7.1f ms, [%"PRId64"]\n", microsecdelay / 1e3, offFrame);
 #endif
-#if 1 // poll 10 times per frame, unless -f delay is given explicitly
-			const long pollinterval = ceilf (nominal_delay * .1f);
+#if 1 // poll 5 times per frame, unless -f delay is given explicitly
+			const long pollinterval = ceilf (nominal_delay * .2f);
 			if (microsecdelay > pollinterval && delay <= 0) microsecdelay = pollinterval;
 #endif
 			if (!select_sleep (microsecdelay)) {
@@ -374,7 +418,7 @@ void event_loop (void) {
 // Video file, rendering and ffmpeg inteface
 //--------------------------------------------
 
-static void render_empty_frame (int blit);
+static void render_empty_frame (int blit, int splashagain);
 static uint8_t displaying_valid_frame = 0;
 
 static int vbufsize = 0;
@@ -400,7 +444,7 @@ void init_moviebuffer (void) {
 		avpicture_fill ((AVPicture *)pFrameFMT, buffer, render_fmt, movie_width, movie_height);
 		pSWSCtx = sws_getContext (pCodecCtx->width, pCodecCtx->height, pCodecCtx->pix_fmt, movie_width, movie_height, render_fmt, SWS_BICUBIC, NULL, NULL, NULL);
 	}
-	render_empty_frame (0);
+	render_empty_frame (0, 0);
 }
 
 void avinit (void) {
@@ -1125,6 +1169,7 @@ static void *index_run (void *arg) {
 	OSD_frame[0] = '\0';
 	OSD_smpte[0] = '\0';
 	osd_smpte_ts = -1;
+	osd_vtc_oob  = -1;
 	index_progress = 0;
 	force_redraw = 1;
 	if (!index_frames()) {
@@ -1498,13 +1543,14 @@ int open_movie (char* file_name) {
 	OSD_nfo_geo[0][sizeof(OSD_nfo_tme[0]) - 1] = '\0';
 
 	current_file = strdup (file_name);
+	x_fib_add_recent (current_file, time (NULL));
 
 	start_index_thread();
 
 	return 0;
 }
 
-static void render_empty_frame (int blit) {
+static void render_empty_frame (int blit, int splashagain) {
 	if (!buffer) return;
 	// clear image (black / or YUV green)
 	if (render_fmt == PIX_FMT_UYVY422) {
@@ -1580,7 +1626,7 @@ static void render_empty_frame (int blit) {
 			buffer[yoff+3]=255;
 		}
 #endif
-	if (!splashed) {
+	if (!splashed || splashagain) {
 		splash(buffer);
 	}
 	if (blit)
@@ -1592,6 +1638,7 @@ void display_frame (int64_t timestamp, int force_update) {
 
 	if (!buffer) {
 		osd_smpte_ts = -1;
+		osd_vtc_oob  = -1;
 		displaying_valid_frame = 0;
 		return;
 	}
@@ -1603,6 +1650,7 @@ void display_frame (int64_t timestamp, int force_update) {
 		OSD_frame[0] = '\0';
 		OSD_smpte[0] = '\0';
 		osd_smpte_ts = -1;
+		osd_vtc_oob  = -1;
 	}
 #endif
 
@@ -1616,12 +1664,13 @@ void display_frame (int64_t timestamp, int force_update) {
 			strcpy(OSD_smpte, syncname[syncnidx]);
 			frame_to_smptestring (&OSD_smpte[4], osd_smpte_ts, 0);
 		}
-		render_empty_frame (need_redisplay);
+		render_empty_frame (need_redisplay, !current_file && !want_nosplash);
 		displaying_valid_frame = 0;
 		return;
 	}
 
 
+	// want_ignstart here ??
 	if (timestamp < 0 || timestamp >= frames) {
 		OSD_frame[0] = '\0';
 		int need_redisplay = force_update || displaying_valid_frame;
@@ -1632,7 +1681,28 @@ void display_frame (int64_t timestamp, int force_update) {
 			strcpy(OSD_smpte, syncname[syncnidx]);
 			frame_to_smptestring (&OSD_smpte[4], osd_smpte_ts, 0);
 		}
-		render_empty_frame (need_redisplay);
+		if ((OSD_mode & (OSD_VTC | OSD_VTCOOR)) == (OSD_VTC | OSD_VTCOOR)) {
+			int64_t oob = timestamp;
+			if (want_ignstart) {
+				oob += file_frame_offset;
+			}
+			if (timestamp < 0) {
+				strcpy(OSD_frame, "-- ");
+				oob -= file_frame_offset;
+			} else {
+				strcpy(OSD_frame, "++ ");
+				oob -= frames;
+			}
+			if (osd_vtc_oob != oob) {
+				need_redisplay = 1;
+			}
+			osd_vtc_oob = oob;
+			frame_to_smptestring (&OSD_frame[3], oob, 1);
+			strcat(OSD_frame, " EOF");
+		}
+		if (need_redisplay) {
+			render_empty_frame (1, 0);
+		}
 		displaying_valid_frame = 0;
 		return;
 	}
@@ -1707,7 +1777,7 @@ void display_frame (int64_t timestamp, int force_update) {
 		// seek failed of no format
 		if (pFrameFMT && want_debug)
 			printf("DEBUG: frame seek unsucessful.\n");
-		render_empty_frame (force_update || displaying_valid_frame);
+		render_empty_frame (force_update || displaying_valid_frame, 0);
 		displaying_valid_frame = 0;
 		last_decoded_pts = -1;
 		last_decoded_frameno = -1;
@@ -1754,6 +1824,8 @@ int close_movie () {
 	OSD_frame[0] = '\0';
 	OSD_smpte[0] = '\0';
 	osd_smpte_ts = -1;
+	osd_vtc_oob  = -1;
 	clear_info();
+	Xletterbox (Xgetletterbox());
 	return (0);
 }
