@@ -99,6 +99,7 @@ enum {
 #define OSD_IDXNFO (0x0800)
 #define OSD_POS    (0x1000)
 #define OSD_GEO    (0x2000)
+#define OSD_VTCOOR (0x4000)
 
 #ifdef TTFFONTFILE
 # define FONT_FILE TTFFONTFILE
@@ -225,6 +226,7 @@ void ui_osd_vtc_tc ();
 void ui_osd_vtc_off ();
 void ui_osd_pos();
 void ui_osd_geo();
+void ui_osd_outofrange ();
 
 enum SyncSource {
 	SYNC_JACK = 0, // used in display_x_dialog.c
@@ -239,7 +241,7 @@ enum SyncSource {
 enum SyncSource ui_syncsource();
 
 /* jack.c function prototypes */
-int64_t jack_poll_frame (void);
+int64_t jack_poll_frame (uint8_t *rolling);
 void open_jack(void );
 void close_jack(void);
 int jack_connected(void);
@@ -286,4 +288,19 @@ int readconfig (char *fn);
 int render_font (char *fontfile, char *text, int px, int dx);
 void free_freetype ();
 
+/* xvesifib.c - shared */
+void x_fib_free_recent ();
+int x_fib_add_recent (const char *path, time_t atime);
+int x_fib_save_recent (const char *fn);
+int x_fib_load_recent (const char *fn);
+unsigned int x_fib_recent_count ();
+const char  *x_fib_recent_at (unsigned int i);
+const char  *x_fib_recent_file(const char *appname);
+
+// configuration
+extern char const * const cfg_features;
+extern char const * const cfg_displays;
+extern char const * const cfg_midi;
+extern char cfg_compat[];
+
 #endif
